#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
# Your imports within this package go here

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################

# Defining once all the tokens used for the compass language.
tokens = {
    "MODULE",  # module keyword.
    "ENDMODULE",  # endmodule keyword.
    "INPUT",  # input keyword.
    "OUTPUT",  # output keyword.
    "NAME",  # string used to identify somathing (probably a signal).
    "EACH",  # each loop keyword.
    "SEQ",  # seq keyword.
    "PAR",  # par keyword.
    "AWAIT",  # await keyword.
    "ASSIGN",  # assignation symbol (<-).
    "EMIT",  # emit keyword.
    "NUMBER",  # literal integer.
    "AND",  # && operator.
    "OR",  # || operator.
    "TESTEQ",  # == operator.
    "LOCAL",  # local keyword.
    "IF",  # The if keyword.
    "ELIF",  # The elif keyword.
    "ELSE",  # The else keyword.
    "ENDIF",  # The endif keyword.
}

# Defining the litteral tokens used in the compass language.
literals = {"(", ")", "{", "}", ";", ",", "+", "-", "/", "%", "*", "!"}

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
import compass.ast as ast  # Used for downcasts

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def codegen_declaration(declaration: ast.Declaration) -> str:
    """
    Outputs the Compass code corresponding to the provided Declaration.
    """
    # We downcast the Declaration to its real type.
    if isinstance(declaration, ast.InputDeclaration):
        return f"input {declaration.signal}"
    elif isinstance(declaration, ast.OutputDeclaration):
        return f"output {declaration.signal}"
    else:
        raise ValueError(f"Invalid declaration type for {declaration}")


##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
import compass.ast as ast  # Used for downcasts
from compass.codegen.compass.module import (
    codegen_module,
)  # Used for recursive codegen

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def codegen_tree(tree: ast.Tree) -> str:
    """
    Outputs the Compass source code for the provided AST.
    """
    # For now an AST only contains one module.
    return codegen_module(tree.module)


##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
from functools import reduce    # Used for elif chains


# External imports
from sly import Parser  # Parser implementation


# Internal imports
import compass.ast as ast  # Used to build the AST
from compass.grammar import compass_tokens  # Used for language tokens

################################### CLASSES ####################################


class CompassParser(Parser):
    """
    Parser for the Compass language, used to build the AST from the tokenized
    source code.
    """

    # DEBUG
    # debugfile = "test/files/parser.out"

    # The tokens for the Compass language.
    tokens = compass_tokens.tokens

    # The only precedence rule is that <- should go before emit.
    precedence = (
        ("nonassoc", IF, ELSE),
        # ELIF is right associative to enable chained statement.
        ("right", ELIF),
        ("nonassoc", MODULE, INPUT, OUTPUT, EACH, EMIT, AWAIT, SEQ, PAR, LOCAL),
        ("left", ","),  # , is left associative, to solve conflicts
        ("left", ";"),  # , is left associative, to solve conflicts
        ("nonassoc", ASSIGN),
        # The next operators are used for expressions.
        ("nonassoc", TESTEQ),
        ("left", OR, "+", "-"),
        ("left", AND, "*", "/", "%"),
        ("right", UMINUS),
        ("right", "!"),
    )

    # The final rule, which returns an AST from the Module.
    @_("module")
    def ast(self, p):
        # Returning an AST from the Module.
        return ast.Tree(p.module)

    # Building a Module from Name + Declarations + Body. The most complex rule
    # in the AST.
    @_('MODULE NAME "(" list_declaration ")" statement ENDMODULE')
    def module(self, p):
        # Returning the Module from the provided description.
        return ast.Module(p.NAME, p.list_declaration, p.statement)

    # Merging two lists of declarations.
    @_('list_declaration "," list_declaration')
    def list_declaration(self, p):
        # Concatenating the two lists.
        return p.list_declaration0 + p.list_declaration1

    # Creating a list of declarations of one element from an input signal.
    @_("INPUT signal")
    def list_declaration(self, p):
        # Building a list of declarations from a single input.
        return [ast.InputDeclaration(p.signal)]

    # Creating a list of declarations of one element from an output signal.
    @_("OUTPUT signal")
    def list_declaration(self, p):
        # Building a list of declarations from a single output.
        return [ast.OutputDeclaration(p.signal)]

    # Creating a loop each statement from a reset Signal + Statement body.
    @_("EACH expression statement")
    def statement(self, p):
        # Creating an Each statement.
        return ast.Each(p.expression, p.statement)

    # Creating a list of statements from a single isolated statement.
    @_("statement")
    def list_statement(self, p):
        return [p.statement]

    # Merging two adjacent lists of statement.
    @_('list_statement ";" list_statement')
    def list_statement(self, p):
        return p.list_statement0 + p.list_statement1

    # Rule to resolve the optionnal last semi-colon for the last statement of a
    # list.
    @_('list_statement ";"')
    def list_statement(self, p):
        return p.list_statement

    # Creating a sequential statement from a scoped list of statements.
    @_('SEQ "{" list_statement "}"')
    def statement(self, p):
        return ast.Seq(p.list_statement)

    # Creating a parallel statement from a scoped list of statement.
    @_('PAR "{" list_statement "}"')
    def statement(self, p):
        return ast.Par(p.list_statement)

    # Create a conditional statement which can be inserted in an elifchain.
    @_("expression statement")
    def list_cond_statement(self, p):
        # Under the hood we just create a new else-if nested level.
        return [ast.IfStatement(p.expression, p.statement)]

    # A chain of conditional statements joined by elif keywords.
    @_("list_cond_statement ELIF list_cond_statement")
    def list_cond_statement(self, p):
        return p.list_cond_statement0 + p.list_cond_statement1

    # Opens a conditional chain with an if and resolves it.
    @_("IF list_cond_statement ENDIF")
    def statement(self, p):
        # Each branch in the conditional chain becomes the else of its
        # predecessor. We iterate over the list backwards since it makes the
        # process more convenient.
        return reduce(
            lambda else_branch, if_branch: if_branch.with_else(else_branch),
            reversed(p.list_cond_statement),
        )

    # Opens a conditional chain with an if and a final else.
    @_("IF list_cond_statement ELSE statement ENDIF")
    def statement(self, p):
        # Each branch in the conditional chain becomes the else of its
        # predecessor. We iterate over the list backwards since it makes the
        # process more convenient. We also use the else statement as a base
        # value.
        return reduce(
            lambda else_branch, if_branch: if_branch.with_else(else_branch),
            reversed(p.list_cond_statement),
            p.statement,
        )

    # Creating a local variable.
    @_("LOCAL NAME")
    def statement(self, p):
        return ast.LocalStatement(p.NAME)

    # Creating an await from a signal.
    @_("AWAIT expression")
    def statement(self, p):
        return ast.AwaitStatement(p.expression)

    # Creating an emit from an assignement.
    @_("EMIT signal ASSIGN expression")
    def statement(self, p):
        return ast.EmitStatement(p.signal, p.expression)

    # Creating an emit from a single signal.
    @_("EMIT signal")
    def statement(self, p):
        # We simply assign 1 to the signal.
        return ast.EmitStatement(p.signal, ast.Number("1"))

    # Creating a Signal from an isolated name.
    @_("NAME")
    def signal(self, p):
        return ast.Signal(p.NAME)

    # Creating an expression from an isolated number.
    @_("NUMBER")
    def expression(self, p):
        return ast.Number(p.NUMBER)

    # Creating an expression from an isolated signal (for complex expressions).
    @_("signal")
    def expression(self, p):
        return ast.SignalExpression(p.signal)

    # Creating the rules for complex expressions which use operators.
    @_("expression TESTEQ expression")
    def expression(self, p):
        return ast.TestEqOp(p.expression0, p.expression1)

    @_("expression OR expression")
    def expression(self, p):
        return ast.OrOp(p.expression0, p.expression1)

    @_('expression "+" expression')
    def expression(self, p):
        return ast.PlusOp(p.expression0, p.expression1)

    @_('expression "-" expression')
    def expression(self, p):
        return ast.MinusOp(p.expression0, p.expression1)

    @_("expression AND expression")
    def expression(self, p):
        return ast.AndOp(p.expression0, p.expression1)

    @_('expression "*" expression')
    def expression(self, p):
        return ast.TimesOp(p.expression0, p.expression1)

    @_('expression "/" expression')
    def expression(self, p):
        return ast.DivOp(p.expression0, p.expression1)

    @_('expression "%" expression')
    def expression(self, p):
        return ast.ModOp(p.expression0, p.expression1)

    # Rules for unary operators.
    @_('"-" expression %prec UMINUS')
    def expression(self, p):
        return ast.UminusOp(p.expression)

    @_('"!" expression')
    def expression(self, p):
        return ast.NotOp(p.expression)

    # Enabling the use of parenthesis to prioritize some operations.
    @_('"(" expression ")"')
    def expression(self, p):
        return p.expression


################################## FUNCTIONS ###################################

# Your functions go here

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
from typing import List  # Used for type hints


# External imports
# Your imports from other packages go here


# Internal imports
from compass.ast.statement import (
    Statement,
)  # Used for type hints and inheritance

################################### CLASSES ####################################


class Seq(Statement):
    """
    Class used to represent the sequential execution of several Statements in
    the AST.
    """

    def __init__(self, list_statement: List[Statement]):
        """
        Constructor of the Seq class.

        Arguments
        =========
         - list_statement: The list of Statements to build this Seq from.
        """
        # Store our arguments.
        self.statements = list_statement


################################## FUNCTIONS ###################################

# Your functions go here

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts: 
# Creation Date: 2021-03-13
# Language: Python3

################################### IMPORTS ####################################

# Standard library 
# Your imports from the standard library go here 


# External imports 
# Your imports from other packages go here 


# Internal imports 
import compass.ast as ast  # Used for downcasts
from compass.codegen.c.declaration import (
    codegen_declaration,
)  # Used for recursive codegen, shared with C target

################################### CLASSES ####################################

# Your classes go here 

################################## FUNCTIONS ###################################

def codegen_module(module: ast.Module) -> str:
    """
    Outputs the header declaration corresponding to the provided Module.
    """
    source_code = ""
    # Adding the name of the module.
    source_code += f"void {module.name}("
    # Building the code for each declaration.
    code_declarations = [
        codegen_declaration(declaration) for declaration in module.declarations
    ]
    # Joining the codes for the declarations.
    source_code += ", ".join(code_declarations)
    # Opening the brackets for the body of the code.
    source_code += ");\n"
    # Returning the prepared source code.
    return source_code

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-13
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
import compass.ast as ast  # Used for downcasts
from compass.codegen.c.module import (
    codegen_module,
)  # Used for recursive codegen

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def codegen_tree(tree: ast.Tree) -> str:
    """
    Outputs the C source code for the provided AST.
    """
    source_code = ""
    # For now an AST only contains one module.
    source_module, owned_states, owned_locals = codegen_module(tree.module)
    # We add some code to statically allocate the states used in the modules.
    # The states have value 0 by default.
    for state in owned_states:
        source_code += f"static int {state} = 0;\n"
    # If there are no states, we add a small comment instead.
    if len(owned_states) == 0:
        source_code += "/* No states for this code. */\n"
    # Creating the clock signal, which always evaluates to True.
    source_code += "\n"
    source_code += f"const int clock_constant = 1;\n"
    source_code += f"const int *clk = &clock_constant;\n"
    # Creating the never signal, which always evaluates to False.
    source_code += f"const int never_constant = 0;\n"
    source_code += f"const int *never = &never_constant;\n"
    source_code += "\n"
    # Creating all the locals as global signals for the code.
    for local in owned_locals:
        source_code += f"static int {local}_value = 0;\n"
        source_code += f"static int *{local} = &{local}_value;\n"
    # If there are no locals, we add a small comment instead.
    if len(owned_locals) == 0:
        source_code += "/* No locals for this code. */\n"
    # Newline for the style.
    source_code += "\n"
    # Adding the code for the inner module.
    source_code += source_module
    # Returning the expected source code.
    return source_code


##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-13
# Language: Python3

################################### IMPORTS ####################################

# Standard library
import argparse  # Used to build the command line interface
import os.path  # Used for path manipulations


# External imports
# Your imports from other packages go here


# Internal imports
from compass.frontend.driver import compass_compile  # The compiler driver

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def command_line():
    """
    Entry point of the command line interface.
    """
    # We first build the CLI using argparse.
    cli_parser = build_frontend_parser()
    # Parsing the command line arguments.
    arguments = cli_parser.parse_args()

    # We read the input source code.
    with open(arguments.SOURCE, "r") as source_file:
        source_code = source_file.read()

    # We call the compiler driver with the right arguments.
    built_code = compass_compile(source_code, arguments.LANG)

    # We write the compiled code to the provided output file. If no output file
    # was provided, we try to use a default one based on the input path and the
    # language used.
    if arguments.OUTPUT == None:
        output_path = default_output(arguments.SOURCE, arguments.LANG)
    else:
        output_path = arguments.OUTPUT

    with open(output_path, "w") as output_file:
        output_file.write(built_code)


def build_frontend_parser() -> argparse.ArgumentParser:
    """
    Builds the CLI parser with argparse.
    """
    cli_parser = argparse.ArgumentParser(prog="compass")
    # Adding the arguments for the frontend.
    cli_parser.add_argument(
        "SOURCE", type=str, help="The source code to compile"
    )
    cli_parser.add_argument(
        "-o",
        "--output",
        dest="OUTPUT",
        default=None,
        type=str,
        help="The file to write the compiled code to",
    )
    cli_parser.add_argument(
        "-l",
        "--lang",
        dest="LANG",
        default="C",
        choices=["compass", "C", "c", "H", "h", "header", "debug"],
        type=str,
        help="The language to compile for",
    )
    # Returning the built parser.
    return cli_parser


def default_output(source_path: str, language: str) -> str:
    """
    Returns the default path to put the output in based on the source_path and
    the language (for the extension).
    """
    # We first get the path of the file without extension.
    path_prefix, _ = os.path.splitext(source_path)
    # Getting the extension based on the language used.
    if language in ["C", "c"]:
        extension = ".c"
    elif language == "compass":
        extension = ".cmps"
    elif language in ["H", "h", "header"]:
        extension = ".h"
    elif language == "debug":
        extension = ".debug.c"
    else:
        raise ValueError(f"Unknown Language {language}")
    # Returning the expected output path.
    return path_prefix + "_compass" + extension


##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
import compass.ast as ast  # Used for downcasts

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def codegen_expression(expression: ast.Expression) -> str:
    """
    Outputs the Compass code corresponding to the provided Expression.
    """
    if isinstance(expression, ast.Number):
        return str(expression)
    elif isinstance(expression, ast.SignalExpression):
        return str(expression.signal)
    elif isinstance(expression, ast.Binary):
        return codegen_binary(expression)
    elif isinstance(expression, ast.Unary):
        return codegen_unary(expression)
    else:
        raise ValueError(f"Unkown Expression {expression}")

def codegen_binary(expression: ast.Binary) -> str:
    """
    Specialized variant of codegen_expression for binary operators.
    """
    # Recursively building the code for the left and right expressions.
    left_code = codegen_expression(expression.left)
    right_code = codegen_expression(expression.right)
    # We downcast to the right operator.
    if isinstance(expression, ast.AndOp):
        return f"({left_code} && {right_code})"
    elif isinstance(expression, ast.ModOp):
        return f"({left_code} % {right_code})"
    elif isinstance(expression, ast.DivOp):
        return f"({left_code} / {right_code})"
    elif isinstance(expression, ast.MinusOp):
        return f"({left_code} - {right_code})"
    elif isinstance(expression, ast.OrOp):
        return f"({left_code} || {right_code})"
    elif isinstance(expression, ast.PlusOp):
        return f"({left_code} + {right_code})"
    elif isinstance(expression, ast.TestEqOp):
        return f"({left_code} == {right_code})"
    elif isinstance(expression, ast.TimesOp):
        return f"({left_code} * {right_code})"
    else:
        raise ValueError(f"Unknown binary operator {expression}")

def codegen_unary(expression: ast.Unary) -> str:
    """
    Specialized variant of codegen_expression for unary operators.
    """
    # Recursively building the code for the inner expression.
    inner_code = codegen_expression(expression.expression)
    # We downcast to the right operator.
    if isinstance(expression, ast.NotOp):
        return f"(!{inner_code})"
    elif isinstance(expression, ast.UminusOp):
        return f"(-{inner_code})"
    else:
        raise ValueError(f"Unknown unary operator {expression}")



##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-13
# Language: Python3

################################### IMPORTS ####################################

# Standard library
from typing import List # Used for type hints.
import os  # Used for path related manipulations.


# External imports
from compass.frontend import compass_compile  # The function we want ot test.


# Internal imports
# Your imports within this package go here

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def collect_examples() -> List[str]:
    """
    Returns the list of the paths of all the example files provided in the
    repository.
    """
    # This can be done in one line.
    return [
        f"examples/{example_file}"
        for example_file in os.listdir("examples")
        if example_file.endswith(".cmps")
    ]


def test_compass_compiler():
    """
    Simple test for the compass_compile function, with compass backend.
    """
    # Collecting the source files.
    examples = collect_examples()
    for example in examples:
        # Reading the source code.
        with open(example, "r") as compass_file:
            compass_source = compass_file.read()
        # Runs the compilation to compass target.
        compass_compiled = compass_compile(compass_source, "compass")
        # Creating the path for the compiled file.
        compiled_path = example.replace("examples/", "test/files/").replace(
            ".cmps", "_compiled.cmps"
        )
        # Writes the output to a file.
        with open(compiled_path, "w") as compass_compiled_file:
            compass_compiled_file.write(compass_compiled)


def test_c_compiler():
    """
    Simple test for the compass_compile function, with C backend.
    """
    # Collecting the source files.
    examples = collect_examples()
    for example in examples:
        # Reading the source code.
        with open(example, "r") as compass_file:
            compass_source = compass_file.read()
        # Runs the compilation to compass target.
        compass_compiled = compass_compile(compass_source, "C")
        # Creating the path for the compiled file.
        compiled_path = example.replace("examples/", "test/files/").replace(
            ".cmps", "_compiled.c"
        )
        # Writes the output to a file.
        with open(compiled_path, "w") as compass_compiled_file:
            compass_compiled_file.write(compass_compiled)


def test_header_compiler():
    """
    Simple test for the compass_compile function, with header backend.
    """
    # Collecting the source files.
    examples = collect_examples()
    for example in examples:
        # Reading the source code.
        with open(example, "r") as compass_file:
            compass_source = compass_file.read()
        # Runs the compilation to compass target.
        compass_compiled = compass_compile(compass_source, "header")
        # Creating the path for the compiled file.
        compiled_path = example.replace("examples/", "test/files/").replace(
            ".cmps", "_compiled.h"
        )
        # Writes the output to a file.
        with open(compiled_path, "w") as compass_compiled_file:
            compass_compiled_file.write(compass_compiled)


def test_debug_compiler():
    """
    Simple test for the compass_compile function, with debug backend.
    """
    # Collecting the source files.
    examples = collect_examples()
    for example in examples:
        # Reading the source code.
        with open(example, "r") as compass_file:
            compass_source = compass_file.read()
        # Runs the compilation to compass target.
        compass_compiled = compass_compile(compass_source, "debug")
        # Creating the path for the compiled file.
        compiled_path = example.replace("examples/", "test/files/").replace(
            ".cmps", "_compiled.debug.c"
        )
        # Writes the output to a file.
        with open(compiled_path, "w") as compass_compiled_file:
            compass_compiled_file.write(compass_compiled)


##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-17
# Language: Python3

################################### IMPORTS ####################################

# Standard library
from typing import List, Optional  # Used for type hints.


# External imports
# Your imports from other packages go here


# Internal imports
# Your imports within this package go here

################################### CLASSES ####################################


class InnerContext:
    """
    Class used to return several interesting values during the recursive
    compilation of compass statements. The variable will often be called "ic" in
    the python code.
    """

    # For some mysterious reasons, the default arguments in the constructor seem
    # to be static, which means shared between the instances of the InnerContext
    # class. That is not at all what I want, hence why I have to dance around
    # with the None here...
    def __init__(
        self,
        source_code: Optional[str] = None,
        immediate_state: Optional[str] = None,
        owned_states: Optional[List[str]] = None,
        owned_locals: Optional[List[str]] = None,
    ):
        """
        Constructor of the InnerContext class.

        Arguments
        =========
         - source_code: The code produced for the inner statement.
         - immediate_state: The immediate state of the Statement returning this
            InnerContext.
         - owned_state: The list of all the states owned by the producer
            statement and all its children.
         - owned_locals: The list of all the local variables owned by this
            statement and its children.
        """
        # Storing the arguments
        if source_code is None:
            self.source_code = ""
        else:
            self.source_code = source_code

        if immediate_state is None:
            self.immediate_state = ""
        else:
            self.immediate_state = immediate_state

        if owned_states is None:
            self.owned_states = list()
        else:
            self.owned_states = owned_states

        if owned_locals is None:
            self.owned_locals = list()
        else:
            self.owned_locals = owned_locals


################################## FUNCTIONS ###################################

# Your functions go here

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts: 
# Creation Date: 2021-03-14
# Language: Python3

################################### IMPORTS ####################################

# Standard library 
# Your imports from the standard library go here 


# External imports 
# Your imports from other packages go here 


# Internal imports 
from compass.ast.binary.binary import Binary
from compass.ast.binary.and_op import AndOp
from compass.ast.binary.div_op import DivOp
from compass.ast.binary.minus_op import MinusOp
from compass.ast.binary.mod_op import ModOp
from compass.ast.binary.or_op import OrOp
from compass.ast.binary.plus_op import PlusOp
from compass.ast.binary.testeq_op import TestEqOp
from compass.ast.binary.times_op import TimesOp

################################### CLASSES ####################################

# Your classes go here 

################################## FUNCTIONS ###################################

# Your functions go here 

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
from setuptools import setup  # Used to build the python package.


# External imports
# Your imports from other packages go here


# Internal imports
# Your imports within this package go here

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################


def main():
    """
    Calls the setup function to define the python package.
    """
    # Reading the long description from the README file.
    with open("README.md", "r") as readme_file:
        readme_content = readme_file.read()

    setup(
        name="roadelou-compass",
        version="0.0.4",
        author="roadelou",
        author_email="",
        packages=[
            "compass",
            "compass/ast",
            "compass/ast/unary",
            "compass/ast/binary",
            "compass/codegen",
            "compass/codegen/compass",
            "compass/codegen/c",
            "compass/codegen/header",
            "compass/codegen/debug",
            "compass/codegen/utils",
            "compass/frontend",
            "compass/grammar",
        ],
        license="GPL3",
        install_requires=["sly"],
        python_requires=">=3.6",
        entry_points="""
        [console_scripts]
        compass=compass.frontend:command_line
        """,
        description="Simple synchronous programming language",
        long_description=readme_content,
        long_description_content_type="text/markdown",
        url="https://github.com/roadelou/compass",
        classifiers=[
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Operating System :: POSIX :: Linux",
            "Topic :: Software Development :: Compilers",
            "Topic :: Software Development :: Code Generators",
            "Topic :: System :: Hardware",
        ]
    )


##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    main()

##################################### EOF ######################################



#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
from compass.ast.module import Module  # Used for type hints.

################################### CLASSES ####################################


class Tree:
    """
    Class used to hold the AST of the compass language.
    """

    def __init__(self, module: Module):
        """
        Constructor of the Tree clas.

        Arguments
        =========
         - module: The Module declared in the current file.
        """
        # Storing the arguments.
        self.module = module


################################## FUNCTIONS ###################################

# Your functions go here

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################
